"""
fritz.api_resources.model
~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: © 2019 by Fritz Labs Incorporated
:license: MIT, see LICENSE for more details.
"""

import fritz
from fritz.api_resources import fritz_object


class Model(fritz_object.FritzObject):
    """Fritz Model object."""

    OBJECT_NAME = "Model"

    @property
    def framework(self):
        """Gets associated framework for model.

        Returns: frameworks.ModelFramework
        """
        if self.model_format == "coreml":
            return fritz.frameworks.CORE_ML
        if self.model_format == "tflite":
            return fritz.frameworks.TENSORFLOW_LITE
        if self.model_format == "tfmobile":
            return fritz.frameworks.TENSORFLOW_MOBILE
        if self.model_format == "keras":
            return fritz.frameworks.KERAS

        return None



"""
fritz.api_resources.model_grade_report
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: © 2019 by Fritz Labs Incorporated
:license: MIT, see LICENSE for more details.
"""
from termcolor import colored

import fritz
from fritz.api_resources import fritz_object
from fritz.api_client import FritzClient


class ModelGradeReport(fritz_object.FritzObject):
    """Model Benchmark report."""

    OBJECT_NAME = "ModelGradeReport"

    @classmethod
    def get(cls, version_id, api_key=None):
        """Get model grade report for version_id

        Args:
            version_id (str): ID of ModelVersion
            api_key (str): Optional API Key

        Returns: ModelGradeReport if report has succeeded.
        """
        url = "/model/version/{version_id}/grader".format(
            version_id=version_id
        )

        client = FritzClient(api_key or fritz.api_key)
        response = client.get(url)
        return fritz.utils.convert_to_fritz_object(response)

    def summary(self):
        """Print summary of model grade report."""
        if self.fail_at:
            self._error_summary()
            return

        self.layer_summary()

        stats = []
        # Check Core ML Compatibility
        incompatible_layers = [
            layer
            for layer in self.layers
            if not layer["is_coremltools_compatible"]
        ]
        compatible = len(incompatible_layers) == 0
        color = "green" if compatible else "red"
        stats.append(("Core ML Compatible", colored(str(compatible), color)))

        # Calculate Predicted Runtime
        if self.predicted_runtime < 1.0:
            runtime = f"{self.predicted_runtime * 1000:.1f} ms"
        else:
            runtime = f"{self.predicted_runtime:.3}s"

        runtime = f"{runtime} ({1 / self.predicted_runtime:,.1f} fps)"
        stats.append(("Predicted Runtime (iPhone X)", runtime))

        # FLOPS, Parameters, Version ID.
        stats.append(("Total MFLOPS", f"{self.total_flops / 1.0e6:,.2f}"))
        stats.append(("Total Parameters", f"{self.total_parameters:,d}"))
        stats.append(("Fritz Version ID", self.version_uid))

        max_title_length = max(len(title) for title, _ in stats)
        summary_str = "Fritz Model Grade Report"
        print("\n" + "-" * len(summary_str))
        print(summary_str)
        print("-" * len(summary_str) + "\n")
        for title, value in stats:
            title = title + ":"
            format_str = "{title:%s}{value}" % str(max_title_length + 5)
            print(format_str.format(title=title, value=value))

    def layer_summary(self):
        """Print summary of layers from report."""
        summary_str = "Model Layer Summary"
        print("\n" + "-" * len(summary_str))
        print(summary_str)
        print("-" * len(summary_str) + "\n")

        layers = [
            (
                "Layer (type)",
                "Output Shape",
                "MFLOPS",
                "Weights",
                "Core ML Compatible",
            )
        ]
        layers.extend(self._summarize_layer(layer) for layer in self.layers)
        max_lengths = [
            max(len(value) for value in values) for values in zip(*layers)
        ]
        for i, fields in enumerate(layers):
            line = []
            for j, field in enumerate(fields):
                format_str = "{value:%s}" % str(max_lengths[j] + 5)
                line.append(format_str.format(value=field))

            line_length = len("".join(line))

            if i == 0:
                print("-" * line_length)
                print("".join(line))
                print("=" * line_length)
            else:
                print("".join(line))
                print("-" * line_length)

    @staticmethod
    def _summarize_layer(layer):
        layer_type = f"{layer['name']} ({layer['layer_cls_name']})"
        output_shape = f"{layer['output_shape']}"
        mflops = f"{layer['flops'] / 1.0e6:,.2f}"
        weights = f"{layer['num_weights']:,d}"
        color = "green" if layer["is_coremltools_compatible"] else "red"
        compatible = colored(layer["is_coremltools_compatible"], color)

        return (layer_type, output_shape, mflops, weights, compatible)

    def _error_summary(self):
        message = "Model Grade Report Failed"
        print(colored(message, "red"))
        print(colored("-" * len(message), "red"))
        print(f"Error Message: {self.error_message}")



"""
fritz.frameworks
~~~~~~~~~~~~~~~~

:copyright: © 2019 by Fritz Labs Incorporated
:license: MIT, see LICENSE for more details.
"""
import io
import os
import tempfile


class ModelFramework(object):
    """Defines a specific model framework"""

    def __init__(self, name, extension, file_cls):
        self.name = name
        self.extension = extension
        self._file_cls = file_cls

    def __hash__(self):
        return hash(self.name)

    def __repr__(self):
        return f"ModelFramework(name={self.name} extension={self.extension})"

    def as_framework_filename(self, filename):
        """Generate filename that representing a model of this framework.

        Args:
            filename (str): filename (i.e. 'mobilenet_v2.h5')

        Returns: str
        """
        file_base = os.path.splitext(os.path.basename(filename))[0]
        return file_base + self.extension

    def to_file(self, model):
        """Build FrameworkFileBase object for Framework.

        Args:
            model: Model

        Returns: FrameworkFileBase
        """
        return self._file_cls(model)

    def build_file(self, path):
        """Build FrameworkFileBase object for a given path.

        Args:
            path: String

        Returns: FrameworkFileBase
        """
        return self._file_cls.build_from_file(path)


class FrameworkFileBase(object):
    """Base class to wrap models for interacting with Fritz."""

    def __init__(self, framework, model):
        self.framework = framework
        self.model = model

    def to_bytes(self):
        """Convert model to bytes.

        Returns: io.BytesIO object
        """
        raise NotImplementedError("Subclass class must implement")

    @classmethod
    def build_from_file(cls, path):
        """Build Framework File from model path.

        Args:
            path (str): Path to file.

        Returns: FrameworkFileBase instance.
        """
        raise NotImplementedError("Subclass class must implement")


class KerasFile(FrameworkFileBase):
    """Wrapper class for Keras model."""

    def __init__(self, model):
        super().__init__(KERAS, model)

    def to_bytes(self):
        # need to actually get the bytes here.
        tmp_file = tempfile.NamedTemporaryFile()
        self.model.save(tmp_file.name)
        tmp_file.seek(0)

        return tmp_file.read()

    @classmethod
    def build_from_file(cls, path):
        """Build KerasFile from model path.

        Args:
            path (str): Path to keras file.

        Returns: KerasFile instance.
        """
        # Keras is imported here so that we don't load keras and tensorflow
        # right away on all fritz imports.
        import keras

        model = keras.models.load_model(path)
        return cls(model)


# Have not implemented build_from_file
# pylint: disable=abstract-method
class CoreMLFile(FrameworkFileBase):
    """Wrapper class for Core ML model."""

    def __init__(self, model):
        super().__init__(CORE_ML, model)

    def to_bytes(self):
        serialized_spec = self.model.get_spec().SerializeToString()
        return io.BytesIO(serialized_spec)


# Have not implemented build_from_file
# pylint: disable=abstract-method
class TensorFlowLiteFile(FrameworkFileBase):
    """Wrapper class for TensorFlow Lite model."""

    def __init__(self, model):
        super().__init__(TENSORFLOW_LITE, model)

    def to_bytes(self):
        return io.BytesIO(self.model)


# Have not implemented build_from_file
# pylint: disable=abstract-method
class TensorFlowMobileFile(FrameworkFileBase):
    """Wrapper class for TensorFlow Mobile model."""

    def __init__(self, model):
        super().__init__(TENSORFLOW_MOBILE, model)

    def to_bytes(self):
        return io.BytesIO(self.model)


TENSORFLOW_LITE = ModelFramework("tflite", ".tflite", TensorFlowLiteFile)
TENSORFLOW_MOBILE = ModelFramework("tfmobile", ".pb", TensorFlowMobileFile)
CORE_ML = ModelFramework("coreml", ".mlmodel", CoreMLFile)
KERAS = ModelFramework("keras", ".h5", KerasFile)


def all_frameworks():
    """List of all supported frameworks.

    Returns: List[ModelFramework]
    """
    return [TENSORFLOW_LITE, TENSORFLOW_MOBILE, CORE_ML, KERAS]


def get_from_filename(filename):
    """Gets the corresponding `ModelFramework` from a filename.

    Args:
        filename (str): Filename.

    Returns: Framework if it is supported.
    """
    for framework in all_frameworks():
        if filename.endswith(framework.extension):
            return framework

    return None


def build_framework_file(path):
    """Builds the framework file from path.

    Args:
        path (str): Path.

    Returns: FrameworkFileBase if it is supported.
    """
    framework = get_from_filename(path)
    if not framework:
        return None

    return framework.build_file(path)



"""
fritz.keras.model_checkpoint
~~~~~~~~~~~~~~~~~~~~~~~~~~~~
This module contains a Keras callback to upload new versions of a
model to Fritz.
"""
import logging

import keras

import fritz

_logger = logging.getLogger(__name__)


class FritzSnapshotCallback(keras.callbacks.Callback):
    """Keras callback to create a ModelSnapshot in Fritz.

    Adding this callback will convert and upload mobile-ready models
    during training.
    """

    def __init__(
        self,
        api_key=None,
        project_id=None,
        model_ids_by_framework=None,
        converters_by_framework=None,
        output_file_name=None,
        period=10,
        deploy=False,
    ):
        # pylint: disable=line-too-long
        """Save a Fritz Snapshot to Fritz.

        Args:
            api_key (Optional[str]): Optional API Key.
            project_id (Optional[str]): Optional project id, required if not
                globally set.
            model_ids_by_framework (Dict[frameworks.ModelFramework,str]):
                Dictionary mapping model framework to model ids. If model_id
                not set for a given platform, a new model will be created.
            converters_by_framework (Dict[frameworks.ModelFramework,Callable[[keras.models.Model], Any]]):
                Dictionary mapping model framework to conversion function.
            output_file_name (str): Name of output_file.
            period (int): Interval (number of epochs) between checkpoints.
            deploy (bool): If True will set active version of model to latest
                uploaded model. Default False.
        """
        super().__init__()
        self._api_key = api_key
        self._project_id = project_id
        self._output_file_name = output_file_name
        self._period = period
        self._deploy = deploy
        self._model_ids = model_ids_by_framework or {}
        self._converters = converters_by_framework or {}

    def add_model_metadata(
        self, logs
    ):  # noqa pylint: disable=unused-argument,no-self-use
        """Adds additional metadata about the model to be stored in Fritz.

        Optionally override this method returning custom information.

        Args:
            logs (dict): Includes values such as `acc` and `loss`.

        Returns: Dict of model metadata.
        """
        return {}

    def on_epoch_end(self, epoch, logs=None):
        """Saves model to Fritz on epoch end.

        Args:
            epoch (int): the epoch number
            logs (dict, optional): logs dict
        """
        _logger.info(f"{self.params.get('epochs')} - {epoch}")
        is_last_epoch = self.params.get("epochs") == (epoch + 1)
        # Adding one so that the very first run does not trigger an upload.
        # If you specify period to be 3, the first upload will be on the 3rd
        # epoch.
        if not is_last_epoch and (epoch + 1) % self._period != 0:
            return

        _logger.info("Creating ModelSnapshot - epoch %s", epoch)
        metadata = {"epoch": epoch, "keras_model_path": self._output_file_name}
        metadata.update(logs or {})
        metadata.update(self.add_model_metadata(logs))
        _, _, models = fritz.ModelSnapshot.create(
            api_key=self._api_key,
            project_id=self._project_id,
            filename=self._output_file_name,
            keras_model=self.model,
            set_active=self._deploy,
            metadata=metadata,
            converters=self._converters,
            model_ids=self._model_ids,
        )

        # Update previously unset model_ids with created models.
        for model in models:
            if not self._model_ids.get(model.framework):
                self._model_ids[model.framework] = model.id



"""
Fritz CLI

usage:
   $ fritz model upload my_keras_file.h5
   $ Model Details
     =============
     Model ID:       6bd0fbb59f264a3c8b79e963558848e2
     Model Name:      multi_person_mobilenet_v1_075_float
     Active version:  1

     Model Version Details
     ======================
     Model Version ID:  420fd0b3ef5e4fc48a5b535244d0c0a3
     Version Number:     1
   $ fritz model benchmark --version-id 420fd0b3ef5e4fc48a5b535244d0c0a3
     ------------------------
     Fritz Model Grade Report
     ------------------------
     Core ML Compatible:              True
     Predicted Runtime (iPhone X):    31.4 ms (31.9 fps)
     Total MFLOPS:                    686.90
     Total Parameters:                1,258,580

:copyright: © 2019 by Fritz Labs Incorporated
:license: MIT, see LICENSE for more details.
"""
import os
from pathlib import Path
import configparser

import click
from termcolor import colored

import fritz
from fritz import configuration

_HOME = str(Path.home())
CONFIG_PATH = os.path.join(_HOME, ".fritz")


class FritzNotConfiguredError(click.UsageError):
    """Error raised when Fritz CLI has not been configured."""

    def __init__(self, ctx=None):
        message = (
            "Please configure Fritz API Key and Project ID:\n"
            "\n  $ fritz config\n"
        )
        super().__init__(message, ctx=ctx)


def update_config(path: str = CONFIG_PATH, **updates):
    """Update Fritz configuration file, creating if it does not exist.

    Args:
        path (str): Path of config file.
        updates (dict): Optional keys to update specified by CLI options.

    Return: ConfigParser
    """
    fritz_config = configuration.load_config_file(path=path)

    if updates:
        try:
            configuration.get_credentials(fritz_config)
        except configparser.NoSectionError:
            defaults = {}

        changes = []
        for key in ["api_key", "project_id"]:
            if key not in updates:
                continue

            original = defaults.get(key)
            new = updates[key]
            changes.append(f"{key}: {original} -> {new}")

        print(
            "\n".join(["Updating config with provided variables.", *changes])
        )

        fritz_config = configuration.update_credentials(
            fritz_config, **updates
        )
        configuration.write_config_file(fritz_config, path=path)
        return fritz_config

    print(
        f"""
    Configure the Fritz CLI.
    ------------------------

    This command will add a ~/.fritz configuration file with your API Key
    and default Project ID.

    The API Key is used to authenticate with the Fritz servers, and the
    Project ID is used to determine whcih project to use when creating and
    benchmarking new models.

    To find your API Key and Project ID, log in to Fritz (https://app.fritz.ai)
    and go to the "Training" tab in you project. Enter the listed API Key and
    Project ID in the following prompts:

    """
    )
    api_key = click.prompt("Fritz API Key", type=str)
    project_id = click.prompt("Fritz Project ID", type=str)

    updates = {"api_key": api_key, "project_id": project_id}
    configuration.update_credentials(fritz_config, **updates)
    configuration.write_config_file(fritz_config, path=path)
    return fritz_config


def _get_config_key(key):
    if key not in ["api_key", "project_id"]:
        return None

    configuration.init_config()
    return getattr(fritz, key, None)


@click.group()
def main():
    """Fritz CLI."""


@main.group(invoke_without_command=True)
@click.pass_context
def config(ctx):
    """Fritz configuration settings.

    Args:
        update: Optional flag to update config file.
    """
    if ctx.invoked_subcommand is not None:
        # Don't want to run default flow if specifically trying to update.
        return

    try:
        configuration.init_config()
    except fritz.errors.InvalidFritzConfigError:
        update_config()

    message = "Fritz Configuration"
    print(message)
    print("=" * len(message))
    print(f"API Key: {fritz.api_key}")
    print(f"Project ID: {fritz.project_id}")


@config.command()
@click.option("--api-key", help="Fritz API Key", default=None)
@click.option("--project-id", help="Fritz Project ID", default=None)
def update(api_key, project_id):
    """Fritz configuration settings.

    Args:
        update: Optional flag to update config file.
    """
    updates = {"api_key": api_key, "project_id": project_id}
    updates = {key: value for key, value in updates.items() if value}
    update_config(**updates)
    fritz.configure()

    message = "Fritz Configuration"
    print(message)
    print("=" * len(message))
    print(f"API Key: {fritz.api_key}")
    print(f"Project ID: {fritz.project_id}")


@main.group()
def model():
    """Commands for working with Fritz Models."""
    try:
        fritz.configure()
    except fritz.errors.InvalidFritzConfigError:
        raise FritzNotConfiguredError()


@model.command()
@click.option("--model-id", default=None)
@click.option("--deploy", is_flag=True, flag_value=True)
@click.option("--api-key", help="Fritz API Key")
@click.option("--project-id", help="Fritz Project ID")
@click.argument("path")
def upload(path, model_id=None, deploy=False, api_key=None, project_id=None):
    """Upload model version to Fritz API.

    Args:
        path: Path to model file.
        model_id: Model ID of existing model.
        deploy: If True, will deploy newly uploaded model.
    """
    filename = os.path.basename(path)
    model_file = fritz.frameworks.build_framework_file(path)
    version, snapshot, fritz_model = fritz.ModelVersion.create(
        filename=filename,
        data=model_file.to_bytes(),
        model_id=model_id,
        set_active=deploy,
        api_key=api_key,
        project_id=project_id,
    )

    print("\nModel Details")
    print("=============")
    print(f"Model ID:       {fritz_model.id}")
    print(f"Model Name:      {fritz_model.name}")
    print(f"Active version:  {fritz_model.active_version}")

    print("\nModel Version Details")
    print("=====================")
    print(f"Model Version ID:  {version.id}")
    print(f"Version Number:     {version.version_number}")

    return version, snapshot, model


@model.command()
@click.option("--version-id", help="Existing model version.")
@click.option("--api-key", help="Fritz API Key")
@click.argument("path", required=False)
@click.pass_context
def benchmark(ctx, version_id=None, path=None, api_key=None):
    """Get model grader report for a model version or Keras file.

    Args:
        version_id: Optional ModelVersion ID.
        path: Optional Path to model
    """
    if not version_id and not path:
        print(colored("Version ID or path to model required.", "yellow"))
        return
    if not version_id:
        version, _, _ = ctx.invoke(upload, path=path)
    else:
        version = fritz.ModelVersion.get(version_id, api_key=api_key)

    report = version.benchmark()
    report.summary()


if __name__ == "__main__":
    main()  # pylint: disable=no-value-for-parameter



"""
fritz.fritz_object
~~~~~~~~~~~~~~~~~~~

:copyright: © 2019 by Fritz Labs Incorporated
:license: MIT, see LICENSE for more details.
"""
import json


class FritzObject(object):
    """An object from the API.

    Subclassed objects are used to define interactions with the API.
    """

    def __init__(self, **kwargs):
        self._data = kwargs

    def __repr__(self):
        identity_parts = [type(self).__name__]

        if isinstance(self.uid, str):
            identity_parts.append("uid=%s" % (self.uid))

        unicode_repr = "<%s> JSON: %s" % (" ".join(identity_parts), str(self))

        return unicode_repr

    def __str__(self):
        return json.dumps(self._data, sort_keys=True, indent=2)

    def __getattr__(self, key):
        if key[0] == "_":
            raise AttributeError(key)

        try:
            return self._data[key]
        except KeyError as err:
            raise AttributeError(*err.args)

    @property
    def id(self):
        """ID of object.

        Returns: str
        """
        # Note: Internally, IDs are stored as UIDs.  However, externally
        # these are consistently referred to as IDs (i.e. model_id).
        return self.uid



"""
fritz.errors
~~~~~~~~~~~~

:copyright: © 2019 by Fritz Labs Incorporated
:license: MIT, see LICENSE for more details.
"""


class FritzError(Exception):
    """Generic Exception class for Fritz Errors."""

    def __init__(self, message, status_code=None):
        """
        Args:
            message (str): Error message
            status_code (Optional[int]): Status Code of response.
        """
        super().__init__(message)
        self._message = message
        self.status_code = status_code

    def __str__(self):
        return self._message or "<empty message>"

    def __repr__(self):
        cls_name = self.__class__.__name__

        return (
            f"{cls_name}(message={self._message} "
            f"status_code={self.status_code})"
        )


class FritzNotInitializedError(FritzError):
    """Error when Python SDK not initialized."""

    def __init__(self):
        message = (
            "`fritz.configure` not called.  Please call "
            "`fritz.configure` with your API Key and Project ID."
        )
        super().__init__(message)


class InvalidFritzConfigError(FritzError):
    """Error when Fritz config contains invalid options."""

    def __init__(self, path):
        message = (
            f"Fritz configuration file at {path} contains invalid options"
        )
        super().__init__(message)



"""
fritz.configuration
~~~~~~~~~~~~~~~~~~~

Helps manage fritz configuration.

:copyright: © 2019 by Fritz Labs Incorporated
:license: MIT, see LICENSE for more details.
"""

import os
from pathlib import Path
import configparser

import fritz
import fritz.errors


_HOME = str(Path.home())
CONFIG_PATH = os.path.join(_HOME, ".fritz")


def init_config(path=CONFIG_PATH):
    """Initializes Fritz configuration, by default checking ~/.fritz.

    If FRITZ_ENV environment variable is set, uses that section in config file.

    Args:
        path (str): Set path to override where config file is initialized from.
    """
    fritz_config = load_config_file(path=path)
    try:
        env = os.environ.get("FRITZ_ENV", "default")
        defaults = dict(fritz_config.items(env))
        fritz.configure(
            api_key=defaults["api_key"],
            project_id=defaults["project_id"],
            api_base=defaults.get("api_base"),
        )
    except (configparser.NoSectionError, KeyError):
        raise fritz.errors.InvalidFritzConfigError(path)


def load_config_file(path=CONFIG_PATH):
    """Loads configuration file from disk.

    Args:
        path (str): Path of config file to load.

    Returns: ConfigParser
    """
    fritz_config = configparser.ConfigParser()
    fritz_config.read(path)
    return fritz_config


def write_config_file(fritz_config, path=CONFIG_PATH):
    """Writes provide configuration file.

    Args:
        fritz_config (ConfigParser): Configuration to save.
        path (str): Path of destination config file.
    """
    fritz_config.write(open(path, "w"))


def get_credentials(fritz_config):
    """Gets credentials from config.

    Args:
        fritz_config (ConfigParser): Fritz Configuration.

    Returns: dict of credentials
    """
    env = os.environ.get("FRITZ_ENV", "default")
    return dict(fritz_config.items(env))


def update_credentials(fritz_config, **updates):
    """Updates Fritz config.

    Args:
        fritz_config (ConfigParser): Fritz Configuration.
        updates (dict): updates

    Returns: ConfigParser
    """
    env = os.environ.get("FRITZ_ENV", "default")
    try:
        defaults = get_credentials(fritz_config)
    except configparser.NoSectionError:
        defaults = {}

    defaults.update(updates)
    fritz_config[env] = defaults

    return fritz_config



# pylint: disable=missing-docstring
import sys
import os
from setuptools import setup
from setuptools import find_packages
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [("pytest-args=", "a", "Arguments to pass into py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ["--boxed"]

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        if isinstance(self.pytest_args, str):
            self.pytest_args = self.pytest_args.split(" ")
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


here = os.path.abspath(os.path.dirname(__file__))
os.chdir(here)


dependency_links = [
    "git+https://github.com/apple/coremltools.git#egg=coremltools"
]


tests_require = [
    "astroid >= 2.1.0",
    "pytest >= 4.0.0",
    "pytest-runner",
    "pytest-pylint",
    "pytest-xdist",
    "requests-mock",
    "pylint",
    "requests_toolbelt",
    "Sphinx",
    "keras",
    "tensorflow",
    "coremltools",
]


setup(
    name="fritz",
    version="1.0.0",
    description="Fritz Machine Learning Library.",
    url="https://github.com/fritzlabs/fritz-python",
    author="Fritz Engineering",
    author_email="engineering@fritz.ai",
    license="MIT",
    zip_safe=False,
    packages=find_packages(),
    dependency_links=dependency_links,
    install_requires=["requests", "click", "termcolor"],
    extras_require={"train": ["keras", "tensorflow"]},
    tests_require=tests_require,
    cmdclass={"test": PyTest},
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    entry_points={"console_scripts": ["fritz = fritz.__main__:main"]},
)
